# Generated by Django 4.1 on 2022-10-02 20:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ext_auth', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='display_name',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='userprofile',
            name='department',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
